/* $Id: xp_udp_probe.c,v 1.2 2002/02/07 17:14:20 mixter Exp $ */
/*
** Copyright (C) 2001 Fyodor Yarochkin <fygrave@tigerteam.net>,
**                    Ofir Arkin       <ofir@sys-security.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** All material for nonprofit, educational use only.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include "xprobe.h"

/*
 packet validation correctness is done within the query sending routine.
 NULL is returned if packet is not valid
 caller has to free() retval->pkt and retval when done (otherwise memory
 leak will occur).
 */


rpack_t *send_udp(struct sockaddr_in to) {

    int sndsock;
    int res;
    int port;
    struct ip *ip;
    struct udphdr *udp;
    unsigned char *pack, *recv_pack;
    int on = 1;
    int packlen;
    rpack_t *retval = NULL;
    
    port = udp_find_closed(to);
    sndsock = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
    setsockopt(sndsock, IPPROTO_IP, IP_HDRINCL, (char *)&on, sizeof(on));
    packlen = sizeof(struct ip) + sizeof(struct udphdr) + UDP_DATA_SIZE;
    pack = (unsigned char *)calloc( packlen, 1);
    ip = (struct ip *)pack;
    udp = (struct udphdr *)(pack + sizeof(struct ip));
    
    ip->ip_p     = IPPROTO_UDP;
    ip->ip_src   = interf.addr;
    ip->ip_dst   = to.sin_addr;
    ip->ip_len   = packlen;
    ip->ip_ttl   = 250;
    ip->ip_v     = IP_VERSION;
    ip->ip_hl    = sizeof(struct ip) >> 2;
#ifdef __linux__
    ip->ip_off   = htons((unsigned short) IP_DF);
#else
    ip->ip_off   = (unsigned short) IP_DF;
#endif    
/* could differ linux from cisco at early stage
 * but confuses us with aix/bsdi/hpux 10.x */
/*    ip->ip_tos  = 0xf8;  */
    ip->ip_tos  = 0x0; 
    ip->ip_id   = rand();

    udp->uh_sport = htons(rand());
    udp->uh_dport = htons(port);
    udp->uh_ulen   = htons((unsigned short) packlen - sizeof(struct ip));
    udp->uh_sum    = udp_cksum_calc(ip, udp, sizeof(struct udphdr) +
                    UDP_DATA_SIZE);
    
#ifdef EBUG
    fprintf(stderr,"TEST: UDP to %s:%i ", inet_ntoa(to.sin_addr)
                                        , port);
#endif
    
   res = sendto(sndsock, (void *)pack, packlen,
                0, (struct sockaddr *)&to, sizeof(struct sockaddr));

   if (res <0) {
       perror("sendto1:");
       close(sndsock);
       free(pack);
       return NULL;
   }
#ifdef EBUG
   fprintf(stderr,"[%d bytes] sent, waiting for reponse.\n", res);
#endif
   recv_pack = read_icmp(&res, ICMP_UNREACH, -1);

   if (recv_pack == NULL || (unsigned)res < sizeof(ip) + 8) {
#ifdef EBUG
       fprintf(stderr,"Receive timeout. Quitting..\n");
#endif
       close(sndsock);
       free(pack);
       return NULL;
   }


  retval=(rpack_t *)calloc(sizeof(rpack_t), 1);

   retval->packsize = res;

   retval->pkt = (char *)calloc(res, 1);
   memcpy((void *)retval->pkt, (void *)recv_pack, res);

   
#ifdef DEBUG
   fprintf(stderr,"Received %d bytes packet.\n", res);
   hexdump(recv_pack, res);
   fprintf(stderr,"\n----\n");
   //hexdump(retval->pkt, res);
#endif    
 
   retval->ip = (struct ip *)retval->pkt;
 /* sanity checks */

#ifdef DEBUG
    fprintf(stderr, "hlen: %d\nlen: %d\n", retval->ip->ip_hl<<2, htons(retval->ip->ip_len));
#endif    
    
   if ((unsigned)res < sizeof(struct ip)      ||
       (retval->ip->ip_hl << 2) > (unsigned)res ||
       ntohs(retval->ip->ip_len) > res ) {
#ifdef EBUG
       fprintf(stderr,"Received maliformed packet!\n");
#endif
       close(sndsock);
       free(pack);
       free(retval->pkt);
       free(retval);
       return NULL;
   }

   retval->icmp = (struct icmp *)(retval->pkt + (retval->ip->ip_hl<<2));

#ifdef DEBUG
   fprintf(stderr,"type: %d code: %d\n", retval->icmp->icmp_type, retval->icmp->icmp_code);
#endif   
          
   retval->orig_pkt = (char *)pack;
   retval->orig_ip = (struct ip *)pack;
   close(sndsock);
   return retval;
}


int udp_find_closed(struct sockaddr_in to) {


    return UDP_PORT; /* lets stick with defines for the moment */

}

